-- --------------------------------------------------------------------
-- **                              Data                              **
-- --------------------------------------------------------------------

EnhancedBars_Version = 1;

local implementation = {
    {
        healthBar = "PlayerFrameHealthBar",
        raise = {
             { PlayerFrame:GetChildren(), 6 },
             { PlayerFrameMyHealPredictionBar, 3 },
             { PlayerFrameOtherHealPredictionBar, 3 },
        },
        unit = "player",
    },
    {
        healthBar = "PetFrameHealthBar",
        raise = {
             { select(2, PetFrame:GetChildren()), 6 },
             { PetFrameMyHealPredictionBar, 3 },
             { PetFrameOtherHealPredictionBar, 3 },
        },
        unit = "pet",
    },
    {
        healthBar = "TargetFrameHealthBar",
        raise = {
             { TargetFrameToT, 6 },
             { TargetFrame:GetChildren(), 6 },
             { TargetFrameMyHealPredictionBar, 3 },
             { TargetFrameOtherHealPredictionBar, 3 },
        },
        unit = "target",
    },
    {
        healthBar = "TargetFrameToTHealthBar",
        raise = {
             { TargetFrameToTTextureFrame, 6 },
        },
        unit = "targettarget",
    },
    {
        healthBar = "FocusFrameHealthBar",
        raise = {
             { FocusFrameToT, 6 },
             { FocusFrameTextureFrame, 6 },
        },
        unit = "focus",
    },
    {
        healthBar = "FocusFrameToTHealthBar",
        raise = {
             { FocusFrameToTTextureFrame, 6 },
        },
        unit = "focustarget",
    },
};

-- Add party frames
do
	for i=1, 4 do
		local frame = _G["PartyMemberFrame"..i];

		tinsert(implementation, {
    			healthBar = frame:GetName().."HealthBar",
			raise = {
				{ select(3, frame:GetChildren()), 5 },
				{ _G[frame:GetName().."MyHealPredictionBar"], 3 },
				{ _G[frame:GetName().."OtherHealPredictionBar"], 3 },
			},
        		unit = "party"..i,
		});

		-- TODO: add pet later
	end
end

-- Add arena frames
do
	-- This defeats the purpose of load-on-demand, but arena frames are quite light
	Arena_LoadUI();

	for i=1, 5 do
		local frame = _G["ArenaEnemyFrame"..i];

		tinsert(implementation, {
    			healthBar = frame:GetName().."HealthBar",
			raise = {
				{ select(2, frame:GetChildren()):GetChildren(), 5 },
			},
        		unit = "arena"..i,
		});

		-- TODO: add pet later
	end
end

-- --------------------------------------------------------------------
-- **                            Functions                           **
-- --------------------------------------------------------------------

function EnhancedBars_InstallHealthBar(healthBar, unit)
    assert(healthBar and unit);
    healthBar.changeBar = EnhancedBars_CreateBar(healthBar, unit, "CHANGE");
    healthBar.absorbBar = EnhancedBars_CreateBar(healthBar, unit, "ABSORB");
    healthBar.necroBar = EnhancedBars_CreateBar(healthBar, unit, "NECRO");
    healthBar.overTimeBar = EnhancedBars_CreateBar(healthBar, unit, "OVERTIME");
end

function EnhancedBars_GetBalance(powerType, unit)
	if OverTimePredictor_GetBalance then
		local showHoT = EnhancedBars_Options["ShowHoT"];
		local showDoT = EnhancedBars_Options["ShowDoT"];
		local total, hot, dot = OverTimePredictor_GetBalance(powerType, unit);

		if showHoT and showDoT then
			return total;
		elseif showHoT then
			return hot;
		elseif showDoT then
			return -dot;
		end
	end
	return 0;
end

function EnhancedBars_GetAbsorb(unit)
	if OverTimePredictor_GetAbsorb then
		local showAbsorb = EnhancedBars_Options["ShowAbsorb"];
		local showNecro = EnhancedBars_Options["ShowNecro"];
		local total, absorb, necro = OverTimePredictor_GetAbsorb(unit);

		if not showAbsorb then absorb = 0; end
		if not showNecro then necro = 0; end
		return absorb, necro;
	end
	return 0, 0;
end

-- --------------------------------------------------------------------
-- **                               Hooks                            **
-- --------------------------------------------------------------------

function EnhancedBars_UnitFrameBar_Update(bar, unit)
    if unit ~= bar.unit then return; end

    local guid = UnitGUID(unit);
    local currentValue = bar:GetValue();
    local maxValue = select(2, bar:GetMinMaxValues());
    local absorb, necro = EnhancedBars_GetAbsorb(unit);
    local changeBar, absorbBar, necroBar, overTimeBar = bar.changeBar, bar.absorbBar, bar.necroBar, bar.overTimeBar;

    if changeBar then
        if changeBar.guid ~= guid then
            EnhancedBars_ResetBar(changeBar);
            changeBar.guid = guid;
        end
        if EnhancedBars_Options["ShowChange"] then
            EnhancedBars_ModifyChangeBar(changeBar, currentValue, maxValue);
        end
    end

    if absorbBar then
        EnhancedBars_ModifyPredictedBar(absorbBar, currentValue + absorb, currentValue, maxValue);
    end

    if necroBar then
        EnhancedBars_ModifyPredictedBar(necroBar, currentValue - necro, currentValue, maxValue);
    end

    if overTimeBar then
        EnhancedBars_ModifyPredictedBar(overTimeBar, currentValue + EnhancedBars_GetBalance(-1, unit), currentValue, maxValue);
    end
end

function EnhancedBars_UnitFrameBar_OnUpdate(bar)
    EnhancedBars_UnitFrameBar_Update(bar, bar.unit);
end

-- --------------------------------------------------------------------
-- **                             Handlers                           **
-- --------------------------------------------------------------------

function EnhancedBars_OnLoad(self)
    hooksecurefunc("UnitFrameHealthBar_Update", EnhancedBars_UnitFrameBar_Update);
    hooksecurefunc("UnitFrameHealthBar_OnUpdate", EnhancedBars_UnitFrameBar_OnUpdate);

    for index, info in pairs(implementation) do
        EnhancedBars_Install(info);
    end
end

function EnhancedBars_Install(info)
    EnhancedBars_InstallHealthBar(_G[info.healthBar], info.unit);

    local raiseList = info.raise;
    for i=1, #raiseList do
        local info = raiseList[i];
        info[1]:SetFrameLevel(info[1]:GetFrameLevel() + info[2]);
    end
end